<?php
/**
 * @package     jtl_janolaw
 * @author      Felix Moche <felix.moche@jtl-software.com
 * @copyright   2015 JTL-Software-GmbH
 */

if (isset($oPlugin->oPluginEinstellungAssoc_arr['janolaw_nutzen']) && $oPlugin->oPluginEinstellungAssoc_arr['janolaw_nutzen'] === 'Y' &&
    !empty($oPlugin->oPluginEinstellungAssoc_arr['janolaw_userid']) && !empty($oPlugin->oPluginEinstellungAssoc_arr['janolaw_shopid'])) {
    if (pruefeALLOWFOPEN()) {
        define('JANOLAW_LOGGING', 0); // Logging, 1 = true, 0 = false
        define('JANOLAW_LOGGING_FILE', PFAD_LOGFILES . 'janolaw.log'); // Logging
        if ($oPlugin->oPluginEinstellungAssoc_arr['janolaw_api_version'] === '3') {
            define('JANOLAW_TYPE_AGB', 'terms');
            define('JANOLAW_TYPE_WRB', 'revocation');
            define('JANOLAW_TYPE_IMP', 'legaldetails');
            define('JANOLAW_TYPE_DAT', 'datasecurity');
            define('JANOLAW_TYPE_WRB_FORM', 'model-withdrawal-form');
        } elseif ($oPlugin->oPluginEinstellungAssoc_arr['janolaw_api_version'] === '2') {
            define('JANOLAW_TYPE_AGB', 'terms');
            define('JANOLAW_TYPE_WRB', 'revocation');
            define('JANOLAW_TYPE_IMP', 'legaldetails');
            define('JANOLAW_TYPE_DAT', 'datasecurity');
            define('JANOLAW_TYPE_WRB_FORM', null);
        } else {
            define('JANOLAW_TYPE_AGB', 'agb');
            define('JANOLAW_TYPE_WRB', 'widerrufsbelehrung');
            define('JANOLAW_TYPE_IMP', 'impressum');
            define('JANOLAW_TYPE_DAT', 'datenschutzerklaerung');
            define('JANOLAW_TYPE_WRB_FORM', null);
        }
        define('JANOLAW_FORMAT_HTML', 'html');
        define('JANOLAW_FORMAT_TEXT', 'txt');
        define('JANOLAW_FORMAT_PDF', 'pdf');
        define('JANOLAW_URL', 'http://www.janolaw.de/agb-service/shops/');

        Jtllog::writeLog('### Plugin Janolaw gestartet => check ...', JTLLOG_LEVEL_DEBUG, false, 'kPlugin', $oPlugin->kPlugin);

        checkJanolaw($oPlugin, intval($oPlugin->oPluginEinstellungAssoc_arr['janolaw_intervall']));
    } else {
        Jtllog::writeLog('Janolaw Update Fehler: allow_url_fopen ist serverseitig nicht aktiv, bitte setzen Sie sich mit Ihrem Hoster in Verbindung.', JTLLOG_LEVEL_ERROR, false, 'kPlugin', $oPlugin->kPlugin);
    }
}

/**
 * @param Plugin $oPlugin
 * @param string $cType
 * @param string $cFormat
 * @param int    $kUpdate
 * @param int    $kSprache
 * @param string $langCode
 */
function getNewJanolaw($oPlugin, $cType, $cFormat, $kUpdate, $kSprache, $langCode)
{
    if ($oPlugin->oPluginEinstellungAssoc_arr['janolaw_api_version'] === '3') {
        if ($langCode === 'ger') {
            $langCode = 'de';
        } elseif ($langCode === 'fra') {
            $langCode = 'fr';
        } else {
            $langCode = 'gb';
        }
        $langCode .= '/';
    } else {
        $langCode = '';
    }

    switch ($cType) {
        case 'AGB':
        case 'terms':
            $cType = JANOLAW_TYPE_AGB;
            break;

        case 'WRB':
        case 'revocation':
            $cType = JANOLAW_TYPE_WRB;
            break;

        case 'Impressum':
        case 'legaldetails':
            $cType = JANOLAW_TYPE_IMP;
            break;

        case 'Datenschutzerklaerung':
        case 'datasecurity':
            $cType = JANOLAW_TYPE_DAT;
            break;

        case 'WRB-Formular':
        case 'model-withdrawal-form':
            $cType = JANOLAW_TYPE_WRB_FORM;
            break;
    }
    //WRB-Form and PDF is only supported with API Version 3
    if ($oPlugin->oPluginEinstellungAssoc_arr['janolaw_api_version'] !== '3' && ($cType === JANOLAW_TYPE_WRB_FORM || $cFormat === JANOLAW_FORMAT_PDF)) {
        return;
    }

    Jtllog::writeLog('Update: kUpdate (Janolaw) ' . $kUpdate . ' - cType ' . $cType . ' - cFormat ' . $cFormat . ' - kSprache ' . $kSprache . ' wird geladen ...', JTLLOG_LEVEL_DEBUG, false, 'kPlugin', $oPlugin->kPlugin);
    // Neue Datei laden
    $cURL = JANOLAW_URL . $oPlugin->oPluginEinstellungAssoc_arr['janolaw_userid'] . '/' . $oPlugin->oPluginEinstellungAssoc_arr['janolaw_shopid'] . '/' . $langCode . $cType;

    if (JANOLAW_FORMAT_TEXT === $cFormat || JANOLAW_FORMAT_PDF === $cFormat) {
        $cURL .=  '.' . $cFormat;
    } else {
        $cURL .= '_include.' . $cFormat;
    }
    Jtllog::writeLog('Hole URL (Janolaw): ' . $cURL, JTLLOG_LEVEL_DEBUG, false, 'kPlugin', $oPlugin->kPlugin);

    $file = @file_get_contents($cURL);

    if ($file !== false) {
        $file     = Shop::DB()->escape($file);
        $cSpalte  = '';
        $cTabelle = '';
        $cWhere   = '';
        switch ($cType) {
            // AGB
            case JANOLAW_TYPE_AGB:
                $cSpalte  = "cAGBContentText";
                $cTabelle = "ttext";
                $cWhere   = " kSprache = " . intval($kSprache);
                if ($cFormat == JANOLAW_FORMAT_HTML) {
                    $cSpalte = "cAGBContentHtml";
                }
                break;

            // WRB
            case JANOLAW_TYPE_WRB:
                $cSpalte  = "cWRBContentText";
                $cTabelle = "ttext";
                $cWhere   = " kSprache = " . intval($kSprache);
                if ($cFormat == JANOLAW_FORMAT_HTML) {
                    $cSpalte = "cWRBContentHtml";
                }
                break;
            // WRB-Formular
            case JANOLAW_TYPE_WRB_FORM:
                if ($cFormat === JANOLAW_FORMAT_HTML) {
                    $cSpalte  = "cContent";
                    $cTabelle = "tlinksprache";
                    $oLink    = Shop::DB()->query("SELECT kLink FROM tlink WHERE nLinkart = " . LINKTYP_WRB_FORMULAR, 1);
                    $cWhere   = "cISOSprache = '" . gibSprachKeyISO("", $kSprache) . "' AND kLink = " . $oLink->kLink;
                }
                break;

            // Impressum
            case JANOLAW_TYPE_IMP:
                $cSpalte  = "cContent";
                $cTabelle = "tlinksprache";
                $oLink    = Shop::DB()->query("SELECT kLink FROM tlink WHERE nLinkart = " . LINKTYP_IMPRESSUM, 1);
                $cWhere   = "cISOSprache = '" . gibSprachKeyISO("", $kSprache) . "' AND kLink = " . $oLink->kLink;
                break;

            // Datenschutz
            case JANOLAW_TYPE_DAT:
                $cSpalte  = "cContent";
                $cTabelle = "tlinksprache";
                $oLink    = Shop::DB()->query("SELECT kLink FROM tlink WHERE nLinkart = " . LINKTYP_DATENSCHUTZ, 1);
                $cWhere   = "cISOSprache = '" . gibSprachKeyISO("", $kSprache) . "' AND kLink = " . $oLink->kLink;
                break;
        }

        Shop::DB()->query(
            "UPDATE " . $cTabelle . "
                SET " . $cSpalte . " = '" . $file . "'
                WHERE " . $cWhere, 3
        );

        Shop::DB()->query(
            "UPDATE xplugin_jtl_janolaw_tupdate
                SET dAktualisiert = now(),
                    nVersuch = 0
                WHERE kUpdate = " . intval($kUpdate), 3
        );
        $now = new DateTime('NOW');
        Shop::Cache()->set('jtljnlw_update', $now->getTimestamp(), array(CACHING_GROUP_PLUGIN, CACHING_GROUP_PLUGIN . '_' . $oPlugin->kPlugin));
        Jtllog::writeLog('(Janolaw) kUpdate: ' . $kUpdate . ' ' . $cType . ' ' . $cFormat . ' wurde aktualisiert.', JTLLOG_LEVEL_DEBUG, false, 'kPlugin', $oPlugin->kPlugin);
    } else {
        Shop::DB()->query(
            "UPDATE xplugin_jtl_janolaw_tupdate
                SET nVersuch = nVersuch + 1
                WHERE kUpdate = " . intval($kUpdate), 3
        );

        Jtllog::writeLog('(Janolaw) kUpdate ' . $kUpdate . ' war nicht erfolgreich.', JTLLOG_LEVEL_ERROR, false, 'kPlugin', $oPlugin->kPlugin);
    }
}

/**
 * Prueft ob DB noch aktuell ist
 *
 * @param Plugin $oPlugin
 * @param int    $nCaching
 */
function checkJanolaw($oPlugin, $nCaching = 24)
{
    $updateTimeStamp = Shop::Cache()->get('jtljnlw_update');
    if ($updateTimeStamp !== false) {
        $lastUpdate = new DateTime();
        $lastUpdate->setTimestamp($updateTimeStamp);
        $now  = new DateTime('NOW');
        $diff = $now->diff($lastUpdate);
        if ((($diff->days * 24) + $diff->h) < intval($nCaching)) {
            return;
        }
    }
    if ($oPlugin->oPluginEinstellungAssoc_arr['janolaw_api_version'] === '3') {
        $langArr = Shop::DB()->query(
            "SELECT kSprache, cISO
                FROM tsprache", 2
        );
    } else {
        $lang = Shop::DB()->query(
            "SELECT kSprache, cISO
                FROM tsprache
                WHERE cISO = 'ger'", 1
        );
        $langArr = array($lang);
    }
    if (is_array($langArr) && count($langArr) > 0) {
        $oUpdate_arr = Shop::DB()->query(
            "SELECT *
                FROM xplugin_jtl_janolaw_tupdate
                WHERE (dAktualisiert = '0000-00-00 00:00:00' OR DATE_ADD(dAktualisiert, INTERVAL " . intval($nCaching) . " HOUR) < now())
                    AND nAktiv = 1", 2
        );

        Jtllog::writeLog('(Janolaw) Text: ' . print_r($oUpdate_arr, true), JTLLOG_LEVEL_DEBUG, false, 'kPlugin', $oPlugin->kPlugin);

        if (is_array($oUpdate_arr) && count($oUpdate_arr) > 0) {
            foreach ($oUpdate_arr as $oUpdate) {
                if (isset($oUpdate->kUpdate) && $oUpdate->kUpdate > 0) {
                    if (isset($oUpdate->nVersuch) && $oUpdate->nVersuch >= 50) {
                        Shop::DB()->query(
                            "UPDATE xplugin_jtl_janolaw_tupdate
                                SET nAktiv = 0
                                WHERE kUpdate = " . intval($oUpdate->kUpdate), 3
                        );
                        Jtllog::writeLog('kUpdate (Janolaw) ' . $oUpdate->kUpdate . ' wurde deaktiviert!', JTLLOG_LEVEL_ERROR, true, 'kPlugin', $oPlugin->kPlugin);
                        continue;
                    }
                    // Updaten
                    foreach ($langArr as $_lang) {
                        getNewJanolaw($oPlugin, $oUpdate->cType, $oUpdate->cFormat, $oUpdate->kUpdate, intval($_lang->kSprache), $_lang->cISO);
                    }
                }
            }
        }
    }
}
