<?php
/**
 * @package     jtl_janolaw
 * @author      Felix Moche <felix.moche@jtl-software.com
 * @copyright   2016 JTL-Software-GmbH
 * @license     http://jtl-url.de/jtlshoplicense
 */

/**
 * Class janolawHelper
 */
class janolawHelper
{
    CONST API_ENDPOINT = 'www.janolaw.de/agb-service/shops/';

    CONST JANOLAW_FORMAT_HTML = 'html';

    CONST JANOLAW_FORMAT_TEXT = 'txt';

    CONST JANOLAW_FORMAT_PDF = 'pdf';

    /**
     * @var array
     */
    var $options = array();

    /**
     * @var Plugin
     */
    var $plugin;

    /**
     * @var array
     */
    var $types = array();

    /**
     * @var null|array
     */
    var $apiVersion = null;

    /**
     * janolawHelper constructor.
     * @param Plugin $oPlugin
     */
    public function __construct(Plugin $oPlugin)
    {
        $this->options = $oPlugin->oPluginEinstellungAssoc_arr;
        $this->plugin  = $oPlugin;
    }

    /**
     *
     */
    public function init()
    {
        $this->apiVersion = $this->getApiVersion();
        $this->setTypes($this->apiVersion['version']);
    }

    /**
     * @param int $apiVersion
     * @return $this
     */
    private function setTypes($apiVersion)
    {
        $this->types['JANOLAW_TYPE_AGB']      = 'terms';
        $this->types['JANOLAW_TYPE_WRB']      = 'revocation';
        $this->types['JANOLAW_TYPE_IMP']      = 'legaldetails';
        $this->types['JANOLAW_TYPE_DAT']      = 'datasecurity';
        $this->types['JANOLAW_TYPE_WRB_FORM'] = 'model-withdrawal-form';

        if ($apiVersion === 2) {
            $this->types['JANOLAW_TYPE_WRB_FORM'] = null;
        } elseif ($apiVersion === 1) {
            $this->types['JANOLAW_TYPE_AGB']      = 'agb';
            $this->types['JANOLAW_TYPE_WRB']      = 'widerrufsbelehrung';
            $this->types['JANOLAW_TYPE_IMP']      = 'impressum';
            $this->types['JANOLAW_TYPE_DAT']      = 'datenschutzerklaerung';
            $this->types['JANOLAW_TYPE_WRB_FORM'] = null;
        }

        return $this;
    }

    /**
     * @param string $type
     * @return string
     */
    public function getDocumentNiceName($type)
    {
        switch ($type) {
            case $this->types['JANOLAW_TYPE_AGB'] :
                return 'Allgemeine Gesch&auml;ftsbedingungen';
            case $this->types['JANOLAW_TYPE_WRB'] :
                return 'Widerrufsbelehrung';
            case $this->types['JANOLAW_TYPE_IMP'] :
                return 'Impressum';
            case $this->types['JANOLAW_TYPE_DAT'] :
                return 'Datenschutzerkl&auml;rung';
            case $this->types['JANOLAW_TYPE_WRB_FORM'] :
                return 'Widerrufs-Formular';
            default :
                return 'Unbekannt';
        }
    }

    /**
     * @param bool $force
     * @return bool
     */
    public function run($force = false)
    {
        $updates = $this->check($this->options['janolaw_intervall'], $force);
        if ($updates !== false) {
            $this->init();
            $apiVersion = $this->getApiVersion();
            $languages  = $this->getLanguages($apiVersion['version'], $apiVersion['multilang']);

            return $this->updateDocuments($languages, $updates, $apiVersion['version']);
        }

        return false;
    }

    /**
     * @param array $languages
     * @param array $documents
     * @param int   $apiVersion
     * @return bool
     */
    private function updateDocuments($languages, $documents, $apiVersion)
    {
        $ok = true;
        foreach ($languages as $language) {
            foreach ($documents as $document) {
                $ok = $ok && $this->update($apiVersion, $document->cType, $document->cFormat, $document->kUpdate, $language->kSprache, $language->cISO);
            }
        }
        if ($ok) {
            $now = new DateTime('NOW');
            Shop::Cache()->set('jtljnlw_update', $now->getTimestamp(), array(CACHING_GROUP_PLUGIN, CACHING_GROUP_PLUGIN . '_' . $this->plugin->kPlugin));
            Shop::Cache()->flushTags(array(CACHING_GROUP_CORE));
        }

        return $ok;
    }

    /**
     * @param int    $apiVersion
     * @param string $cType
     * @param string $cFormat
     * @param int    $kUpdate
     * @param int    $kSprache
     * @param string $langCode
     * @return bool
     */
    private function update($apiVersion, $cType, $cFormat, $kUpdate, $kSprache, $langCode)
    {
        $kLink = 0;
        if ($apiVersion === 1) {
            $langCode = '';
        }
        switch ($cType) {
            case 'AGB':
            case 'terms':
                $cType = $this->types['JANOLAW_TYPE_AGB'];
                break;

            case 'WRB':
            case 'revocation':
                $cType = $this->types['JANOLAW_TYPE_WRB'];
                break;

            case 'Impressum':
            case 'legaldetails':
                $oLink = Shop::DB()->select('tlink', 'nLinkart', LINKTYP_IMPRESSUM);
                if (!isset($oLink->kLink)) {
                    return false;
                }
                $kLink = (int)$oLink->kLink;
                $cType = $this->types['JANOLAW_TYPE_IMP'];
                break;

            case 'Datenschutzerklaerung':
            case 'datasecurity':
                $oLink = Shop::DB()->select('tlink', 'nLinkart', LINKTYP_DATENSCHUTZ);
                if (!isset($oLink->kLink)) {
                    return false;
                }
                $kLink = (int)$oLink->kLink;
                $cType = $this->types['JANOLAW_TYPE_DAT'];

                break;

            case 'WRB-Formular':
            case 'model-withdrawal-form':
                $oLink = Shop::DB()->select('tlink', 'nLinkart', LINKTYP_WRB_FORMULAR);
                if (!isset($oLink->kLink)) {
                    return false;
                }
                $kLink = (int)$oLink->kLink;
                $cType = $this->types['JANOLAW_TYPE_WRB_FORM'];
                break;
        }
        //WRB-Form and PDF is only supported with API Version 3
        if ($apiVersion < 3 && ($cType === $this->types['JANOLAW_TYPE_WRB_FORM'] || $cFormat === self::JANOLAW_FORMAT_PDF)) {
            return false;
        }
        $data           = new stdClass();
        $data->format   = $cFormat;
        $data->langCode = $langCode;
        $data->type     = $cType;

        $res = http_get_contents($this->getApiUrl($data));

        if (!empty($res)) {
            $update = new stdClass();
            $keys   = array();
            $values = array();
            $table  = '';

            switch ($cType) {
                // AGB
                case $this->types['JANOLAW_TYPE_AGB'] :
                    $table    = 'ttext';
                    $keys[]   = 'kSprache';
                    $values[] = (int)$kSprache;
                    if ($cFormat === self::JANOLAW_FORMAT_HTML) {
                        $update->cAGBContentHtml = $res;
                    } else {
                        $update->cAGBContentText = $res;
                    }
                    break;

                // WRB
                case $this->types['JANOLAW_TYPE_WRB'] :
                    $table    = 'ttext';
                    $keys[]   = 'kSprache';
                    $values[] = (int)$kSprache;
                    if ($cFormat === self::JANOLAW_FORMAT_HTML) {
                        $update->cWRBContentHtml = $res;
                    } else {
                        $update->cWRBContentText = $res;
                    }
                    break;
                // WRB-Formular
                case $this->types['JANOLAW_TYPE_WRB_FORM'] :
                    if (version_compare(JTL_VERSION, 405, '>=')) {
                        //since shop 4.05 the agb/wrb backend knows about wrb forms
                        $table    = 'ttext';
                        $keys[]   = 'kSprache';
                        $values[] = $kSprache;
                        if ($cFormat === self::JANOLAW_FORMAT_HTML) {
                            $update->cWRBFormContentHtml = $res;
                        } else {
                            $update->cWRBFormContentText = $res;
                        }
                    } elseif ($cFormat === self::JANOLAW_FORMAT_HTML) {
                        //for older shop version we use the link type and can only use html
                        $update->cContent = $res;
                        $table    = 'tlinksprache';
                        $keys[]   = 'cISOSprache';
                        $keys[]   = 'kLink';
                        $values[] = gibSprachKeyISO('', $kSprache);
                        $values[] = $kLink;
                    }
                    break;

                // Impressum
                case $this->types['JANOLAW_TYPE_IMP'] :
                    $update->cContent = $res;
                    $table    = 'tlinksprache';
                    $keys[]   = 'cISOSprache';
                    $keys[]   = 'kLink';
                    $values[] = gibSprachKeyISO('', $kSprache);
                    $values[] = $kLink;
                    break;

                // Datenschutz
                case $this->types['JANOLAW_TYPE_DAT'] :
                    $update->cContent = $res;
                    $table    = 'tlinksprache';
                    $keys[]   = 'cISOSprache';
                    $keys[]   = 'kLink';
                    $values[] = gibSprachKeyISO('', $kSprache);
                    $values[] = $kLink;
                    break;

                default :
                    return false;
            }

            if (!empty($table)) {
                Shop::DB()->update($table, $keys, $values, $update);
                $update                = new stdClass();
                $update->dAktualisiert = 'now()';
                $update->nVersuch      = 0;
                Shop::DB()->update('xplugin_jtl_janolaw_tupdate', 'kUpdate', (int)$kUpdate, $update);
                Jtllog::writeLog('(janolaw) kUpdate: ' . (int)$kUpdate . ' ' . $cType . ' ' . $cFormat . ' wurde aktualisiert.', JTLLOG_LEVEL_DEBUG, false, 'kPlugin', (int)$this->plugin->kPlugin);

                return true;
            }
        } else {
            Shop::DB()->query(
                "UPDATE xplugin_jtl_janolaw_tupdate
                    SET nVersuch = nVersuch + 1
                    WHERE kUpdate = " . (int)$kUpdate, 3
            );

            Jtllog::writeLog('(janolaw) kUpdate ' . (int)$kUpdate . ' war nicht erfolgreich.', JTLLOG_LEVEL_ERROR, false, 'kPlugin', (int)$this->plugin->kPlugin);
        }

        return false;
    }

    /**
     * @param int  $apiVersion
     * @param bool $multilang
     * @return array
     */
    private function getLanguages($apiVersion, $multilang = false)
    {
        $where     = ($apiVersion === 3 && $multilang === true) ?
            '' :
            " WHERE cISO = 'ger'";
        $languages = Shop::DB()->query("
          SELECT kSprache, cISO
            FROM tsprache" . $where, 2
        );
        foreach ($languages as $language) {
            if ($language->cISO === 'ger') {
                $language->cISO = 'de';
            } elseif ($language->cISO === 'fre') {
                $language->cISO = 'fr';
            } else {
                $language->cISO = 'gb';
            }
        }

        return $languages;
    }

    /**
     * determines the user API version
     *
     * @return array
     */
    public function getApiVersion()
    {
        if ($this->apiVersion !== null) {
            return $this->apiVersion;
        }
        $testData           = new stdClass();
        $testData->format   = self::JANOLAW_FORMAT_PDF;
        $testData->langCode = 'de';
        $testData->type     = 'legaldetails';

        $res = http_get_contents($this->getApiUrl($testData));

        if (!empty($res)) {
            $testData->langCode = 'gb';
            $testData->type     = 'legaldetails';
            $testData->format   = self::JANOLAW_FORMAT_HTML;
            $res                = http_get_contents($this->getApiUrl($testData));

            $this->apiVersion = array(
                'status'    => 'OK',
                'version'   => 3,
                'multilang' => !empty($res)
            );

            return $this->apiVersion;
        }
        $testData->format = self::JANOLAW_FORMAT_HTML;
        $testData->type   = 'legaldetails';

        $res = http_get_contents($this->getApiUrl($testData));
        if (!empty($res)) {
            $this->apiVersion = array(
                'status'    => 'OK',
                'version'   => 2,
                'multilang' => false
            );

            return $this->apiVersion;
        }

        $testData->langCode = '';
        $testData->format   = self::JANOLAW_FORMAT_HTML;
        $testData->type     = 'impressum';

        $res = http_get_contents($this->getApiUrl($testData));
        $this->apiVersion = (empty($res)) ?
            array('status' => 'FAILED', 'version' => 0, 'multilang' => false) :
            array('status' => 'OK', 'version' => 1, 'multilang' => false);

        return $this->apiVersion;
    }

    /**
     * @param stdClass $data
     * @return string
     */
    private function getApiUrl($data)
    {
        $data->format = ($data->format === self::JANOLAW_FORMAT_TEXT || $data->format === self::JANOLAW_FORMAT_PDF) ?
            ('.' . $data->format) :
            ('_include.' . $data->format);
        if (!empty($data->langCode)) {
            $data->langCode .= '/';
        }
        $protocol = (defined('JTL_JANOLAW_USE_SSL') && JTL_JANOLAW_USE_SSL === true) ?
            'https://' :
            'http://';

        return $protocol . self::API_ENDPOINT .
            trim($this->options['janolaw_userid']) . '/' .
            trim($this->options['janolaw_shopid']) . '/' .
            $data->langCode . $data->type . $data->format;
    }

    /**
     * @param int  $hours
     * @param bool $force
     * @return array|bool
     */
    public function check($hours = 24, $force = false)
    {
        $hours           = (int)$hours;
        $updateTimeStamp = Shop::Cache()->get('jtljnlw_update');
        if (!$force && $updateTimeStamp !== false) {
            $lastUpdate = new DateTime();
            $lastUpdate->setTimestamp($updateTimeStamp);
            $now  = new DateTime('NOW');
            $diff = $now->diff($lastUpdate);
            if ((($diff->days * 24) + $diff->h) < intval($hours)) {
                return false;
            }
        }
        $where = ($force) ?
            '' :
            "(dAktualisiert = '0000-00-00 00:00:00' OR DATE_ADD(dAktualisiert, INTERVAL " . $hours . " HOUR) < now()) AND ";
        $oUpdate_arr = Shop::DB()->query("
                SELECT *
                  FROM xplugin_jtl_janolaw_tupdate
                    WHERE " . $where . "nAktiv = 1", 2
        );

        if (is_array($oUpdate_arr) && count($oUpdate_arr) > 0) {
            foreach ($oUpdate_arr as $oUpdate) {
                if (isset($oUpdate->kUpdate) && $oUpdate->kUpdate > 0) {
                    if (isset($oUpdate->nVersuch) && $oUpdate->nVersuch >= 50) {
                        Shop::DB()->query(
                            "UPDATE xplugin_jtl_janolaw_tupdate
                                    SET nAktiv = 0
                                    WHERE kUpdate = " . (int)$oUpdate->kUpdate, 3
                        );
                        Jtllog::writeLog('kUpdate (janolaw) ' . (int)$oUpdate->kUpdate . ' wurde deaktiviert!',
                            JTLLOG_LEVEL_ERROR, true, 'kPlugin', (int)$this->plugin->kPlugin);
                        return false;
                    }
                }
            }

            return $oUpdate_arr;
        }

        return false;
    }

    /**
     * @return array
     */
    public function getPdfDownloadLinks()
    {
        $pdfs = array();
        $this->init();
        if ($this->apiVersion['version'] > 2) {
            $languages  = $this->getLanguages($this->apiVersion['version'], $this->apiVersion['multilang']);
            $data = new stdClass();
            foreach ($languages as $language) {
                foreach ($this->types as $document) {
                    $data->format   = self::JANOLAW_FORMAT_PDF;
                    $data->langCode = $language->cISO;
                    $data->type     = $document;
                    $pdfs[] = array('type' => $data->type, 'niceName' => $this->getDocumentNiceName($data->type), 'format' => $data->format, 'langCode' => $data->langCode, 'url' => $this->getApiUrl($data));
                }
            }
        }

        return $pdfs;
    }

    /**
     * @return bool
     */
    public function isConfigured()
    {
        return (!empty($this->options['janolaw_userid']) && !empty($this->options['janolaw_shopid']));
    }

    /**
     * @return array|int|object
     */
    public function getStatus()
    {
        if (!$this->isConfigured()) {
            return array();
        }
        $links = Shop::DB()->query("
            SELECT nLinkart 
              FROM tlink 
              WHERE nLinkart IN (" . LINKTYP_IMPRESSUM . ", " . LINKTYP_AGB . ", " . LINKTYP_WRB . ", " . LINKTYP_WRB_FORMULAR . ", " . LINKTYP_DATENSCHUTZ . ") 
        ", 2);
        $pages = array();
        foreach ($links as $link) {
            $pages[] = (int)$link->nLinkart;
        }
        $oUpdate_arr = Shop::DB()->query("
            SELECT *
                FROM xplugin_jtl_janolaw_tupdate
                ORDER BY kUpdate", 2
        );
        foreach ($oUpdate_arr as $update) {
            $update->niceName = $this->getDocumentNiceName($update->cType);
            switch ($update->cType) {
                case $this->types['JANOLAW_TYPE_AGB'] :
                    $update->pageExists = in_array(LINKTYP_AGB, $pages);
                    break;
                case $this->types['JANOLAW_TYPE_WRB'] :
                    $update->pageExists = in_array(LINKTYP_WRB, $pages);
                    break;
                case $this->types['JANOLAW_TYPE_IMP'] :
                    $update->pageExists = in_array(LINKTYP_IMPRESSUM, $pages);
                    break;
                case $this->types['JANOLAW_TYPE_DAT'] :
                    $update->pageExists = in_array(LINKTYP_DATENSCHUTZ, $pages);
                    break;
                case $this->types['JANOLAW_TYPE_WRB_FORM'] :
                    $update->pageExists = in_array(LINKTYP_WRB_FORMULAR, $pages);
                    break;
                default:
                    $update->pageExists = false;
                    break;
            }
        }

        return $oUpdate_arr;
    }
}
